{
open Parser;;
open Lineno;;

let cdepth = ref 0;;
let lit = ref "";;

exception LexFail of string
}

let alpha = ['A'-'Z' 'a'-'z']
let upper = ['A'-'Z']
let lower = ['a'-'z']
let digit = ['0'-'9']

let id = (alpha)(alpha|digit|'''|'_')*
let ws = [' ''\t']

let char = (alpha)|(digit)|'('|')'|'{'|'}'|'['|']'|'.'|' '|','|'_'|'-'|'/'|'\\'|':'|'?'

rule lex =
  parse
        ['('][':']            { cdepth := !cdepth + 1;
				comment lexbuf }
      | [':'][')']            { raise (LexFail "unmatched close comment")}
      | ['\n']                { Lineno.incr (); lex lexbuf }
      | ws+                   { lex lexbuf }
      | digit+                { INT(int_of_string(Lexing.lexeme lexbuf)) }
      | "-"digit+             { INT(int_of_string(Lexing.lexeme lexbuf)) }
      | "="                   { EQ }
      | "in"                  { IN }
      | "["                   { LBRACK }
      | "]"                   { RBRACK }
      | "{"                   { LBRACE }
      | "}"                   { RBRACE }
      | "("                   { LPAREN }
      | ")"                   { RPAREN }
      | "."                   { DOT }
      | ","                   { COMMA }
      | "->"                  { ARROW }
      | "@"                   { AMP }
      | ":"                   { COLON }
      | ";"                   { SEMI }
      | "_"                   { UNDERSCORE }
      | "~"                   { TILDE }
      | "*"                   { STAR }
      | "+"                   { PLUS }
      | "?"                   { QUESTION }
      | "/"                   { SLASH }
      | "::"                  { CONS }
      | ":="                  { ASSIGN }
      
      | "type"                { TYPE }
      |	"#quit"               { QUIT }

      | "left"                { LEFT }
      | "right"               { RIGHT }
      | "child"               { CHILD }
      | "children"            { CHILDREN }
      | "iterate"             { ITERATE }
      | "snapshot"            { SNAPSHOT }
      | "for"                 { FOR }
      | "let"                 { LET }
      | "skip"                { SKIP }
      | "if"                  { IF }
      | "then"                { THEN }
      | "else"                { ELSE }
      | "in"                  { IN }
      | "insert"              { INSERT }
      | "delete"              { DELETE }
      | "bool"                { BOOL }
      | "string"              { STRING }
      | "return"              { RETURN }
      | "true"                { TRUE }
      | "false"               { FALSE }
      | "not"                 { NOT }
      |	"\'"char"\'"          { CHAR(String.get (Lexing.lexeme lexbuf) 1) }
      | "\'\\\"\'"            { CHAR('\"') }
      | "\'\\n\'"             { CHAR('\n') }
      | "\'\\t\'"             { CHAR('\t') }
      | "\'\\\\\'"            { CHAR('\\') }
      | "\'\\\'\'"            { CHAR('\'') }
      | "\""                  { literal lexbuf }
      | id                    { ID(Lexing.lexeme lexbuf) }
      | "$"id                 { VAR(Lexing.lexeme lexbuf) }
      | "$_"id                { TVAR(Lexing.lexeme lexbuf) }
      | eof                   { EOF }
      | _                     { raise (LexFail "illegal character")}


and comment =
parse
        ['('][':']            { cdepth := !cdepth + 1; comment lexbuf }
      |	[':'][')']            { cdepth := !cdepth - 1;
				if !cdepth = 0 
				then lex lexbuf
				else comment lexbuf} 
      | ['\n']                { Lineno.incr (); comment lexbuf }
      | _                     { comment lexbuf }
      | eof                   { raise (LexFail "unclosed comment") } 
      



and literal = 
parse 
        char                  { lit := !lit ^ Lexing.lexeme lexbuf;
			        literal lexbuf }
      |	"\\\n"                { literal lexbuf }
      | "\\\""                { lit := !lit ^ "\"" ; literal lexbuf }
      | "\\n"                 { lit := !lit ^ "\n" ; literal lexbuf }
      | "\\t"                 { lit := !lit ^ "\t" ; literal lexbuf }
      | "\\\\"                { lit := !lit ^ "\\" ; literal lexbuf }
      | "\\\'"                { lit := !lit ^ "\'" ; literal lexbuf }
      | "\""                  { let s = !lit in  lit := ""; LITERAL(s) }

{
}
