open Dtd;;
open Muxq;;
open Update;;
open Xml;;

let parse data =
	match data.[0] with
	| '#' -> Xml.parse_file (String.sub data 1 ((String.length data)-2))
	| _ -> Xml.parse_string data

;;


let parse_query_from_channel ch = 
  Parser.query_file Lexer.lex (Lexing.from_channel ch);;
let parse_update_from_channel ch = 
  Parser.update_file Lexer.lex (Lexing.from_channel ch);;


let rec output_xml_seq ch xs = 
  let output_endline s = output_string ch (s^"\n") in
  match xs with 
    [] -> output_endline "()\n"
  | [x] -> output_endline (Xml.to_string_fmt x)
  | x::xs -> output_endline(Xml.to_string_fmt x ^ ",");
      output_xml_seq ch xs
;;

let print_xml_seq = output_xml_seq stdout;;

exception TooManyFiles of string;;

let main args = 
  try
    let infile = ref None in
    let updfile = ref None in
    let quiet = ref false in
    let outfile = ref None in
    let set_filename kind filename s = 
      match !filename with 
	None -> filename := Some s
      | Some _ -> raise (TooManyFiles kind) in
    let spec = 
      [("-in",Arg.String(set_filename "input" infile),"Set input file");
       ("-upd",Arg.String(set_filename "update" updfile),"Set update file");
       ("-out", Arg.String(set_filename "output" outfile), "Set output file");
       ("-q", Arg.Set(quiet), "Quiet mode")] in
    Arg.parse spec (set_filename "input" infile) "usage: lux [options] files";
    let x = 
      match !infile with
      | Some fn ->  [Xml.parse_file fn]
      | None -> [] in 
    if not(!quiet) then print_endline "Input value...";
    if not(!quiet) then print_xml_seq x;
    let update = (
      match !updfile with
	Some fn -> let inch = open_in fn in
	let update = parse_update_from_channel inch in
	close_in inch;
	update
      | None -> Skip
     ) in
    if not(!quiet) then (
      print_endline "Update statement...";
      print_endline (Printer.pp Update.pp update);
      print_endline "Running update..."
     );
    let z = exec empty_env x update in
    if not(!quiet) then print_xml_seq z;
    match !outfile with
      None -> ()
    | Some f -> 
	let outch = open_out f in
	output_xml_seq outch z;
	close_out outch
  with 
    TooManyFiles kind -> print_endline ("Too many "^kind^" files specified")
  | Parsing.Parse_error -> 
      print_endline("Syntax error near ["^Lineno.filename()^":"
		    ^(string_of_int (Lineno.lineno()))^"]\n")
  | Lexer.LexFail msg -> 
      print_endline("Error near line ["^Lineno.filename()^":"
		    ^(string_of_int (Lineno.lineno()))^"]: "^msg^"\n") 
;;

main Sys.argv;;
