%{
open Muxq;;
open Update;;
%}

%token <string> ID LITERAL VAR TVAR
%token <int> INT
%token <char> CHAR
%token LPAREN RPAREN LBRACK RBRACK LBRACE RBRACE 
%token COLON UNDERSCORE
%token ARROW COMMA SEMI AMP EQ DOT UNDERSCORE TILDE SLASH
%token STAR PLUS QUESTION
%token FOR LET IN NOT TRUE FALSE TYPE RETURN ASSIGN
%token IF THEN ELSE
%token SNAPSHOT LEFT RIGHT CHILD CHILDREN CONS ITERATE SKIP
%token INSERT DELETE REPLACE STRING BOOL
%token QUIT 
%token EOF 

%type <Muxq.exp> query 
%type <Update.stmt> update

%type <Muxq.exp> query_file
%type <Update.stmt> update_file

%start  query_file
%start  update_file
%start  query
%start  update

%%

query_file : query EOF                  { $1 }
update_file : update EOF                  { $1 }

query  
: query0 COMMA query                    { Seq($1,$3) }
| query0 EQ query                       { Eq($1,$3) }
| query0                                { $1 }
| FOR TVAR IN query RETURN query        { For($2,$4,$6) }
| LET VAR ASSIGN query IN query         { Let($2,$4,$6) }
;;


query0 
: ID LBRACK query RBRACK                 { Tree($1,$3) }
| ID LBRACK RBRACK                       { Tree($1,Empty) }
| LITERAL                                { String($1) }
| LPAREN RPAREN                          { Empty }
| VAR                                    { Var($1) }
| TVAR                                   { Tvar($1) }
| TVAR SLASH ID                          { NodeTest(Child($1),$3) }
| TVAR SLASH STAR                        { Child($1) }
| TVAR SLASH CHILD CONS ID               { NodeTest(Child($1),$5) }
| TVAR SLASH CHILD CONS STAR             { Child($1) }
| LPAREN query RPAREN                    { $2 }
;;


update
: update0                               { $1 }
| update SEMI update0                   { SeqStmt($1,$3) }
;;

update0
: update1                               { $1 }
| IF query THEN update0 ELSE update0    { IfThenElseStmt($2,$4,$6) }
| SNAPSHOT VAR IN update0               { Snapshot($2,$4) }
| LET VAR ASSIGN query IN update0       { LetStmt($2,$4,$6) }
| ID QUESTION update0                   { LabelTest ($1,$3) }
| STRING QUESTION update0               { StringTest ($3) }
| LEFT update0                          { Left($2) }
| RIGHT update0                         { Right($2) }
| ITERATE update0                       { Iterate($2) }
| CHILDREN update0                      { Children($2) }
;;
 

update1 
: LBRACE update RBRACE                  { $2 }
| INSERT query                          { Insert($2) }
| DELETE                                { Delete }
| SKIP                                  { Skip }
;;
